unit csLogClass;

{
  CzySkored-Base coded by sk0r / Czybik

  Features:
    - OpenGL
    - Exporttable
    - Usermsg
    - Events
    - Enginefuncs
    - PreS_DynamicSound
    - Log Class
    - Ini Class
    - Menu Class
    - Useful routines

  File: csLogClass.pas
  Description: Log Class for file logging

  Be sure to read the Readme.txt for more information
}

interface

uses
  SysUtils,
  Windows;

type
  TLog = class
      constructor Create(szFileName: String);
      procedure Free;
    private
      lpFileName: String;
      dwLogCount: Cardinal;
      procedure RewriteLog(lpBuf: String);
    public
      procedure ClearLog;
      procedure InsertBlankLine;
      procedure InsertLine(lpLineString: String; bInsertTime: Boolean = true);
      procedure InsertLineFmt(lpLineString: String; fmt: array of const; bInsertTime: Boolean = true);
      procedure DeleteLine(lpLineCount: Cardinal);
  end;

implementation

constructor TLog.Create(szFileName: String);
var
  hFile: TextFile;
begin
  inherited Create;

  lpFileName := szFileName;
  dwLogCount := 3;

  if FileExists(lpFileName) then
    DeleteFile(PChar(lpFileName));

  AssignFile(hFile, lpFileName);
  ReWrite(hFile);
  WriteLn(hFile, 'Skored-Base logfile created at ' + timetostr(time));
  WriteLn(hFile, '**********************************************');
  WriteLn(hFile, '');
  CloseFile(hFile);
end;

procedure TLog.Free;
begin
  if FileExists(lpFileName) then
    DeleteFile(PChar(lpFileName));
end;

procedure TLog.RewriteLog(lpBuf: String);
var
  hFile: TextFile;
begin
  if FileExists(lpFileName) then
    DeleteFile(PChar(lpFileName));

  AssignFile(hFile, lpFileName);
  ReWrite(hFile);
  Write(hFile, lpBuf);
  CloseFile(hFile);
end;

procedure TLog.ClearLog;
var
  g: Cardinal;
begin
  for g := 3 to dwLogCount do
    DeleteLine(g);

  dwLogCount := 3;
end;

procedure TLog.InsertBlankLine;
var
  hFile: TextFile;
begin
  AssignFile(hFile, lpFileName);
  Append(hFile);
  WriteLn(hFile, '');
  CloseFile(hFile);
  Inc(dwLogCount, 1);
end;

procedure TLog.InsertLine(lpLineString: String; bInsertTime: Boolean = true);
var
  hFile: TextFile;
begin
  AssignFile(hFile, lpFileName);
  Append(hFile);
  if bInsertTime then
    WriteLn(hFile, '[' + timetostr(time) + '] ' + lpLineString)
  else
    WriteLn(hFile, lpLineString);
  CloseFile(hFile);
  Inc(dwLogCount, 1);
end;

procedure TLog.InsertLineFmt(lpLineString: String; fmt: array of const; bInsertTime: Boolean = true);
var
  hFile: TextFile;
  formatString: String;
begin
  AssignFile(hFile, lpFileName);
  Append(hFile);
  formatString := Format(lpLineString, fmt);
  if bInsertTime then
    WriteLn(hFile, '[' + timetostr(time) + '] ' + formatString)
  else
    WriteLn(hFile, formatString);
  CloseFile(hFile);
  inc(dwLogCount, 1);
end;

procedure TLog.DeleteLine(lpLineCount: Cardinal);
var
  hFile: TextFile;
  szLine: String;
  szWhole: String;
  dwCount: Cardinal;
begin
  dwCount := 1;
  szWhole := '';
  AssignFile(hFile, lpFileName);
  ReSet(hFile);
  while not eof(hFile) do
  begin
    ReadLn(hFile, szLine);
    if dwCount <> lpLineCount then
      szWhole := szWhole + szLine + #13#10;
    Inc(dwCount);
  end;
  CloseFile(hFile);
  RewriteLog(szWhole);
  Dec(dwLogCount, 1);
end;

end.