unit csMenuClass;

{
  CzySkored-Base coded by sk0r / Czybik

  Features:
    - OpenGL
    - Exporttable
    - Usermsg
    - Events
    - Enginefuncs
    - PreS_DynamicSound
    - Log Class
    - Ini Class
    - Menu Class
    - Useful routines

  File: csMenuClass.dpr
  Description: Menu Class code

  Be sure to read the Readme.txt for more information
}

interface

//====================================================================
uses
  SysUtils,
  uEngFuncs;
//====================================================================


//====================================================================
const MENU_KEY_UP = 129;
const MENU_KEY_DOWN = 128;
const MENU_KEY_PRESS = 13;
//====================================================================


//====================================================================
type
  TMenuAddr = procedure;

type
  TMenuEntry = record
    szName: String;
    dwID: Cardinal;
    pCvar: PInteger;
    pAddr: TMenuAddr;
  end;

type
  TMenuColor = record
    r, g, b, a: Byte;
  end;

type
  TCzyMenu = class
      constructor Create(lpHeadline: String; x, y: Integer; colBorder, colHeadline, colBody: TMenuColor; t: Integer; iEngOffset: Cardinal);
    private
      pEngFuncs: cl_enginefuncs_s;
      lpMenuHeadline: String;
      XPos: Integer;
      YPos: Integer;
      iWidth: Integer;
      iHeight: Integer;
      bBorder: TMenuColor;
      bHeadline: TMenuColor;
      bBody: TMenuColor;
      MenuBuf: array of TMenuEntry;
    public
      dwMenuCount: Cardinal;
      dwEntrys: Cardinal;
      procedure AddMenuEntry(lpName: String; lpCvar: PInteger; pFuncAddr: TMenuAddr);
      procedure SwitchToEntry(dwEntry: Cardinal);
      procedure DrawMenu;
      procedure KeyEvent(keynumb: Integer);
  end;
//====================================================================

implementation

//====================================================================
constructor TCzyMenu.Create(lpHeadline: String; x, y: Integer; colBorder, colHeadline, colBody: TMenuColor; t: Integer; iEngOffset: Cardinal);
begin
  pEngFuncs := cl_enginefuncs_s(iEngOffset);
  lpMenuHeadline := lpHeadline;
  XPos := x;
  YPos := y;
  iWidth := t;
  iHeight := 20;
  bBorder := colBorder;
  bHeadline := colHeadline;
  bBody := colBody;
  dwEntrys := 0;
  SetLength(MenuBuf, 1);
end;
//====================================================================


//====================================================================
procedure TCzyMenu.AddMenuEntry(lpName: String; lpCvar: PInteger; pFuncAddr: TMenuAddr);
begin
  iHeight := iHeight + 10;
  Inc(dwEntrys);
  SetLength(MenuBuf, dwEntrys+1);
  MenuBuf[dwEntrys].szName := lpName;
  MenuBuf[dwEntrys].dwID := dwEntrys;
  MenuBuf[dwEntrys].pCvar := lpCvar;
  MenuBuf[dwEntrys].pAddr := pFuncAddr;
end;
//====================================================================


//====================================================================
procedure TCzyMenu.SwitchToEntry(dwEntry: Cardinal);
begin
  if dwEntry < dwEntrys+1 then
    dwMenuCount := dwEntry;
end;
//====================================================================


//====================================================================
procedure TCzyMenu.DrawMenu;
var
  g: Byte;
begin
  pEngFuncs.pfnFillRGBA(XPos-2, YPos-1, 2, iHeight+1, bBorder.r, bBorder.g, bBorder.b, bBorder.a); //Left border
  pEngFuncs.pfnFillRGBA(XPos+iWidth, YPos-1, 2, iHeight+1, bBorder.r, bBorder.g, bBorder.b, bBorder.a); //Right border
  pEngFuncs.pfnFillRGBA(XPos-1, YPos-1, iWidth+1, 2, bBorder.r, bBorder.g, bBorder.b, bBorder.a); //Top border
  pEngFuncs.pfnFillRGBA(XPos, YPos+15, iWidth, 2, bBorder.r, bBorder.g, bBorder.b, bBorder.a); //Middle border
  pEngFuncs.pfnFillRGBA(XPos-2, YPos+iHeight, iWidth+4, 2, bBorder.r, bBorder.g, bBorder.b, bBorder.a); //Bottom border
  pEngFuncs.pfnFillRGBA(XPos, YPos+1, iWidth, 14, bHeadline.r, bHeadline.g, bHeadline.b, bHeadline.a); //Top fill
  pEngFuncs.pfnFillRGBA(XPos, YPos+17, iWidth, iHeight-17, bBody.r, bBody.g, bBody.b, bBody.a); //Bottom fill

  pEngFuncs.pfnDrawConsoleString(XPos+2, YPos, PChar(lpMenuHeadline));

  for g := 1 to dwEntrys do
  begin
    if dwMenuCount = g then
      pEngFuncs.pfnDrawSetTextColor(0.0, 0.0, 0.0)
    else
      pEngFuncs.pfnDrawSetTextColor(0.5, 0.5, 0.25);
    pEngFuncs.pfnDrawConsoleString(XPos+2, YPos + 7 + (g*10), PChar(MenuBuf[g].szName + ': ' + inttostr(MenuBuf[g].pCvar^)));
  end;
end;
//====================================================================


//====================================================================
procedure TCzyMenu.KeyEvent(keynumb: Integer);
begin
  if keynumb = MENU_KEY_UP then
    Inc(dwMenuCount, 1);
  if keynumb = MENU_KEY_DOWN then
    Dec(dwMenuCount, 1);

  if keynumb = MENU_KEY_PRESS then
    if @MenuBuf[dwMenuCount].pAddr <> nil then
      MenuBuf[dwMenuCount].pAddr;

  if dwMenuCount < 1 then dwMenuCount := dwEntrys;
  if dwMenuCount > dwEntrys then dwMenuCount := 1;
end;
//====================================================================

end.
