unit uEngFuncs;
{***
*
*	Copyright (c) 1999, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****}
// Simple translation to Delphi Language by DeepBlueSea and sk0r
// please keep in mind that this source is restricted to non-commerical use and
// in fact owned by Valve LLC.

interface
  uses Windows;

{$IFDEF FPC}
{$PACKRECORDS C}
{$ENDIF}

//***************************************************************************//
//                                                                           //
//                               Constants                                   //
//                                                                           //
//***************************************************************************//
const
  STUDIO_RENDER = 1;
  STUDIO_EVENTS = 2;
  MAX_CLIENTS = 32;
  MAX_EDICTS = 900;
  MAX_MODEL_NAME = 64;
  MAX_MAP_HULLS = 4;
  MIPLEVELS = 4;
  NUM_AMBIENTS = 4;
  MAXLIGHTMAPS = 4;
  PLANE_ANYZ = 5;
  ALIAS_Z_CLIP_PLANE = 5;
  ALIAS_LEFT_CLIP = $0001;
  ALIAS_TOP_CLIP = $0002;
  ALIAS_RIGHT_CLIP = $0004;
  ALIAS_BOTTOM_CLIP = $0008;
  ALIAS_Z_CLIP = $0010;
  ALIAS_ONSEAM = $0020;
  ALIAS_XY_CLIP_MASK = $000F;
  
const
  HISTORY_MAX	 = 64;
  IN_ATTACK = 1 shl 0;
  IN_JUMP = 1 shl 1;
  IN_DUCK = 1 shl 2;
  IN_FORWARD = 1 shl 3;
  IN_BACK = 1 shl 4;
  IN_USE = 1 shl 5;
  IN_CANCEL = 1 shl 6;
  IN_LEFT = 1 shl 7;
  IN_RIGHT = 1 shl 8;
  IN_MOVELEFT = 1 shl 9;
  IN_MOVERIGHT = 1 shl 10;
  IN_ATTACK2 = 1 shl 11;
  IN_RUN = 1 shl 12;
  IN_RELOAD = 1 shl 13;
  IN_ALT1 = 1 shl 14;
  IN_SCORE = 1 shl 15;

const
  FL_ONGROUND = 1 shl 9;
  FL_DUCKING  = 1 shl 14;
  FL_INWATER  = 1 shl 4;

const
  FEVENT_ORIGIN = 1 shl 0;
  FEVENT_ANGLES = 1 shl 1;

const
	MAX_PHYSENTS = 600;
	MAX_MOVEENTS = 64;
	MAX_CLIP_PLANES = 5;
	PM_NORMAL = $00000000;
	PM_STUDIO_IGNORE = $00000001;
	PM_STUDIO_BOX = $00000002;
	PM_GLASS_IGNORE = $00000004;
	PM_WORLD_ONLY = 00000008;
	PM_TRACELINE_ANYVISIBLE = 0;
	PM_TRACELINE_PHYSENTSONLY = 1;
  MAX_PHYSINFO_STRING = 256;

const
  kRenderFxNone = 0;
  kRenderFxPulseSlow = 1;
	kRenderFxPulseFast = 2;
	kRenderFxPulseSlowWide = 3;
	kRenderFxPulseFastWide = 4;
	kRenderFxFadeSlow = 5;
	kRenderFxFadeFast = 6;
	kRenderFxSolidSlow = 7;
	kRenderFxSolidFast = 8;
	kRenderFxStrobeSlow = 9;
	kRenderFxStrobeFast = 10;
	kRenderFxStrobeFaster = 11;
	kRenderFxFlickerSlow = 12;
	kRenderFxFlickerFast = 13;
	kRenderFxNoDissipation = 14;
	kRenderFxDistort = 15;
	kRenderFxHologram = 16;
	kRenderFxDeadPlayer = 17;
	kRenderFxExplode = 18;
	kRenderFxGlowShell = 19;
	kRenderFxClampMinScale = 20;

// Studio Render lighting options
const
  STUDIO_NF_FLATSHADE		= 1;
  STUDIO_NF_CHROME = 2;
  STUDIO_NF_FULLBRIGHT = 4;

//***************************************************************************//
//                                                                           //
//                                VarTypes                                   //
//                                                                           //
//***************************************************************************//
type
  LongIntArr4 = array[0..3] of longint;
  CharArr16   = array[0..15] of char;
  HSPRITE = Integer;
  in2float = Array of Array of Single;
//***************************************************************************//
//                                                                           //
//                               Structures                                  //
//                                                                           //
//***************************************************************************//

type
  TRICULLSTYLE = (TRI_FRONT = 0,TRI_NONE = 1);

type
  vec3_s = ^vec3_t;
  vec3_t =  record
    x,y,z: single;
  end;  

type
  rect_s = ^wrect_t;
  wrect_t = record
    left, right, top, bottom: Integer;
  end;

type
  client_sprite_s = ^client_sprite_t;
  client_sprite_t = record
    szName  : Array[0..63] of Char;
    szSprite: Array[0..63] of Char;
    hspr    : Integer;
    iRes    : Integer;
    rc      : wrect_t;
  end;

type
  SCREENINFO_s = ^SCREENINFO;
  SCREENINFO = record
    iSize       : Integer;
    iWidth      : Integer;
    iHeight     : Integer;
    iFlags      : Integer;
    iCharHeight : Integer;
    charWidths  : Array[0..255] of ShortInt;
  end;

type
  TScreenPos = record
    X : Integer;
    Y: Integer;
  end;

type
  cvar_s = ^cvar_t;
  cvar_t = record
    name    : PChar;
    _string : PChar;
    flags   : Integer;
    value   : Single;
    next    : cvar_s;
  end;

type
  PPOINT = ^tagPOINT;
  tagPOINT = record
    x : LongInt;
    y : LongInt;
   end;

type
  hud_player_info_s = ^hud_player_info_t;
  hud_player_info_t = packed record
    name          : PChar;
    ping          : array[0..2] of Byte;
    thisplayer    : Byte;
    spectator     : Byte;
    packetloss    : Byte;
    adummy,bdummy : Byte;
    model         : PChar;
    topcolor      : array[0..2] of Byte;
    bottomcolor   : array[0..2] of Byte;
  end;

type
  client_textmessage_s = ^client_textmessage_t;
  client_textmessage_t = record
    effect   : LongInt;
    r1       : Byte;
    g1       : Byte;
    b1       : Byte;
    a1       : Byte;
    r2       : Byte;
    g2       : Byte;
    b2       : Byte;
    a2       : Byte;
    x        : single;
    y        : single;
    fadein   : single;
    fadeout  : single;
    holdtime : single;
    fxtime   : single;
    pName    : Pchar;
    pMessage : Pchar;
  end;

type
  con_nprint_s = ^con_nprint_t;
  con_nprint_t = record
    index         : Integer;
    time_to_live  : single;
    color         : Array[0..2] of single;
  end;

type
  color24 = record
    r,g,b: byte;
  end;    

type
  qboolean = Integer;

type
  entity_state_s = ^entity_state_t;
  entity_state_t = record
    entityType     : longint;
    number         : longint;
    msg_time       : single;
    messagenum     : longint;
    origin         : vec3_t;
    angles         : vec3_t;
    modelindex     : longint;
    sequence       : longint;
    frame          : single;
    colormap       : longint;
    skin           : smallint;
    solid          : smallint;
    effects        : longint;
    scale          : single;
    eflags         : byte;
    rendermode     : longint;
    renderamt      : longint;
    rendercolor    : color24;
    renderfx       : longint;
    movetype       : longint;
    animtime       : single;
    framerate      : single;
    body           : longint;
    controller     : array[0..3] of byte;
    blending       : array[0..3] of byte;
    velocity       : vec3_t;
    mins           : vec3_t;
    maxs           : vec3_t;
    aiment         : longint;
    owner          : longint;
    friction       : single;
    gravity        : single;
    team           : longint;
    playerclass    : longint;
    health         : longint;
    spectator      : qboolean;
    weaponmodel    : longint;
    gaitsequence   : longint;
    basevelocity   : vec3_t;
    usehull        : longint;
    oldbuttons     : longint;
    onground       : longint;
    iStepLeft      : longint;
    flFallVelocity : single;
    fov            : single;
    weaponanim     : longint;
    startpos       : vec3_t;
    endpos         : vec3_t;
    impacttime     : single;
    starttime      : single;
    iuser1         : longint;
    iuser2         : longint;
    iuser3         : longint;
    iuser4         : longint;
    fuser1         : single;
    fuser2         : single;
    fuser3         : single;
    fuser4         : single;
    vuser1         : vec3_t;
    vuser2         : vec3_t;
    vuser3         : vec3_t;
    vuser4         : vec3_t;
 end;

type
  position_history_t = record
    animtime: single;
    origin  : vec3_t;
    angles  : vec3_t;
  end;

type
  mouth_t = record
    mouthopen : byte;
    sndcount : byte;
    sndavg : longint;
   end;

type
  latchedvars_t = record
    prevanimtime    : single;
    sequencetime    : single;
    prevseqblending : array[0..1] of byte;
    prevorigin      : vec3_t;
    prevangles      : vec3_t;
    prevsequence    : longint;
    prevframe       : single;
    prevcontroller  : array[0..3] of byte;
    prevblending    : array[0..1] of byte;
  end;

type
  colorVec = record
    r : dword;
    g : dword;
    b : dword;
    a : dword;
  end;

type
  modtype_t  = (mod_brush,mod_sprite,mod_alias,mod_studio);
  synctype_t = (ST_SYNC = 0,ST_RAND);

type
  dmodel_s = ^dmodel_t;
  dmodel_t = record
    mins      : array[0..2] of single;
    maxs      : array[0..2] of single;
    origin    : array[0..2] of single;
    headnode  : array[0..(MAX_MAP_HULLS)-1] of longint;
    visleafs  : longint;
    firstface : longint;
    numfaces  : longint;
  end;

type
  mplane_s = ^mplane_t;
  mplane_t = record
    normal   : vec3_t;
    dist     : single;
    _type    : byte;
    signbits : byte;
    pad      : array[0..1] of byte;
  end;

type
  mvertex_t = record
    position : vec3_t;
  end;

type
  medge_t = record
    v                : array[0..1] of word;
    cachededgeoffset : dword;
  end;

type
  texture_s = ^texture_t;
  texture_t = record
    name            : array[0..15] of char;
    width           : dword;
    height          : dword;
    anim_total      : longint;
    anim_min        : longint;
    anim_max        : longint;
    anim_next       : texture_s;
    alternate_anims : texture_s;
    offsets         : array[0..(MIPLEVELS)-1] of dword;
    paloffset       : dword;
  end;

type
  mtexinfo_s = ^mtexinfo_t;
  mtexinfo_t = record
    vecs : array[0..1] of array[0..3] of single;
    mipadjust : single;
    texture : ^texture_t;
    flags : longint;
  end;


type
  mnode_s = ^mnode_t;
  mnode_t = record
    contents     : longint;
    visframe     : longint;
    minmaxs      : array[0..5] of smallint;
    parent       : mnode_s;
    plane        : mplane_s;
    children     : array[0..1] of mnode_s;
    firstsurface : word;
    numsurfaces  : word;
  end;

type
  msurface_s = ^msurface_t;
  msurface_t = record
    visframe    : longint;
    dlightframe : longint;
    dlightbits  : longint;
    plane       : mplane_s;
    flags       : longint;
    firstedge   : longint;
    numedges    : longint;
    cachespots  : array[0..(MIPLEVELS)-1] of Pointer; //^surfcache_s;
    texturemins : array[0..1] of smallint;
    extents     : array[0..1] of smallint;
    texinfo     : mtexinfo_s;
    styles      : array[0..(MAXLIGHTMAPS)-1] of byte;
    samples     : ^color24;
    pdecals     : Pointer; //^decal_t;
  end;

type
  decal_s = ^decal_t;
  decal_t = record
    pnext       : decal_s;
    psurface    : msurface_s;
    dx          : smallint;
    dy          : smallint;
    texture     : smallint;
    scale       : byte;
    flags       : byte;
    entityIndex : smallint;
  end;

type
  mleaf_s = ^mleaf_t;
  mleaf_t = record
    contents            : longint;
    visframe            : longint;
    minmaxs             : array[0..5] of smallint;
    parent              : mnode_s;
    compressed_vis      : ^byte;
    efrags              : Pointer; //^efrag_s;
    firstmarksurface    : ^msurface_s;
    nummarksurfaces     : longint;
    key                 : longint;
    ambient_sound_level : array[0..(NUM_AMBIENTS)-1] of byte;
  end;

type
  dclipnode_t = record
    planenum : longint;
    children : array[0..1] of smallint;
  end;

type
  hull_s = ^hull_t;
  hull_t = record
    clipnodes     : ^dclipnode_t;
    planes        : ^mplane_t;
    firstclipnode : longint;
    lastclipnode  : longint;
    clip_mins     : vec3_t;
    clip_maxs     : vec3_t;
  end;

type
  cache_user_s = ^cache_user_t;
  cache_user_t = record
    data : pointer;
  end;

type
  model_s = ^model_t;
  model_t = record
    name : array[0..(MAX_MODEL_NAME)-1] of char;
    needload          : qboolean;
    _type             : modtype_t;
    numframes         : longint;
    synctype          : synctype_t;
    flags             : longint;
    mins              : vec3_t;
    maxs              : vec3_t;
    radius            : single;
    firstmodelsurface : longint;
    nummodelsurfaces  : longint;
    numsubmodels      : longint;
    submodels         : dmodel_s;
    numplanes         : longint;
    planes            : ^mplane_t;
    numleafs          : longint;
    leafs             : ^mleaf_s;
    numvertexes       : longint;
    vertexes          : ^mvertex_t;
    numedges          : longint;
    edges             : ^medge_t;
    numnodes          : longint;
    nodes             : ^mnode_t;
    numtexinfo        : longint;
    texinfo           : ^mtexinfo_t;
    numsurfaces       : longint;
    surfaces          : ^msurface_t;
    numsurfedges      : longint;
    surfedges         : ^longint;
    numclipnodes      : longint;
    clipnodes         : ^dclipnode_t;
    nummarksurfaces   : longint;
    marksurfaces      : ^msurface_s;
    hulls             : array[0..(MAX_MAP_HULLS)-1] of hull_t;
    numtextures       : longint;
    textures          : ^texture_s;
    visdata           : ^byte;
    lightdata         : ^color24;
    entities          : Pchar;
    cache             : cache_user_t;
  end;

type
  Singles2 = packed record
    x,y: Single;
  end;

type
  triangleapi_s = ^triangleapi_t;
  triangleapi_t = record
    version       : longint;
    RenderMode    : procedure (mode:longint);cdecl;
    _Begin        : procedure (primitiveCode:longint);cdecl;
    _End          : procedure ;cdecl;
    Color4f       : procedure (r:single; g:single; b:single; a:single);cdecl;
    Color4ub      : procedure (r:byte; g:byte; b:byte; a:byte);cdecl;
    TexCoord2f    : procedure (u:single; v:single);cdecl;
    Vertex3fv     : procedure (worldPnt:Psingle);cdecl;
    Vertex3f      : procedure (x:single; y:single; z:single);cdecl;
    Brightness    : procedure (brightness:single);cdecl;
    CullFace      : procedure (style:TRICULLSTYLE);cdecl;
    SpriteTexture : function  (pSpriteModel:model_s; frame:longint):longint;cdecl;
    WorldToScreen : function  (world:PSingle; screen: PSingle):longint;cdecl;
    Fog           : procedure (flFogColor:array of single; flStart:single; flEnd:single; bOn:longint);cdecl;
  end;  

type
  cl_entity_s = ^cl_entity_t;
  cl_entity_t = record
    index             : longint;
    player            : qboolean;
    baseline          : entity_state_t;
    prevstate         : entity_state_t;
    curstate          : entity_state_t;
    current_position  : longint;
    ph                : array[0..(HISTORY_MAX)-1] of position_history_t;
    mouth             : mouth_t;
    latched           : latchedvars_t;
    lastmove          : single;
    origin            : vec3_t;
    angles            : vec3_t;
    attachment        : array[0..3] of vec3_t;
    trivial_accept    : longint;
    model             : model_s;
    efrag             : Pointer;
    topnode           : Pointer;
    syncbase          : single;
    visframe          : longint;
    cvFloorColor      : colorVec;
end;

type
  usercmd_s = ^usercmd_t;
  usercmd_t = record
    lerp_msec       : smallint;
    msec            : byte;
    viewangles      : vec3_t;
    forwardmove     : single;
    sidemove        : single;
    upmove          : single;
    lightlevel      : byte;
    buttons         : smallint;
    impulse         : byte;
    weaponselect    : byte;
    impact_index    : longint;
    impact_position : vec3_t;
  end;

type
  event_args_s = ^event_args_t;
  event_args_t = record
    flags       : longint;
    entindex    : longint;
    origin      : array[0..2] of single;
    angles      : array[0..2] of single;
    velocity    : array[0..2] of single;
    ducking     : longint;
    fparam1     : single;
    fparam2     : single;
    iparam1     : longint;
    iparam2     : longint;
    bparam1     : longint;
    bparam2     : longint;
  end;


type
 pmplane_t = record
   normal : vec3_t;
   dist   : single;
 end;

type
  pmtrace_s = ^pmtrace_t;
  pmtrace_t = record
    allsolid      : qboolean;
    startsolid    : qboolean;
    inopen        : qboolean;
    inwater       : qboolean;
    fraction      : single;
    endpos        : vec3_t;
    plane         : pmplane_t;
    ent           : longint;
    deltavelocity : vec3_t;
    hitgroup      : longint;
  end;

type
  link_s = ^link_t;
  link_t = packed record
    prev, next : link_s;
  end;

type
  string_t = Integer;

type
  screenfade_s = ^screenfade_t;
  screenfade_t = record
    fadeSpeed : single;
    fadeEnd : single;
    fadeTotalEnd : single;
    fadeReset : single;
    fader : byte;
    fadeg : byte;
    fadeb : byte;
    fadealpha : byte;
    fadeFlags : longint;
  end;

type
  physent_s = ^physent_t;
  physent_t = record
	  name: array[0..32] of char;
  	player: integer;
	  origin: vec3_t;
	  model: model_s;
	  studiomodel: model_s;
	  mins, maxs: vec3_t;
	  info: integer;
	  angles: vec3_t;
	  solid: integer;
	  skin: integer;
	  rendermode: integer;
	  frame: single;
	  sequence: integer;
	  controller: array[0..4] of byte;
	  blending: array[0..2] of byte;
	  movetype: integer;
	  takedamage: integer;
	  blooddecal: integer;
	  team: integer;
	  classnumber: integer;
	  iuser1: integer;
	  iuser2: integer;
	  iuser3: integer;
	  iuser4: integer;
	  fuser1: single;
	  fuser2: single;
	  fuser3: single;
	  fuser4: single;
	  vuser1: vec3_t;
	  vuser2: vec3_t;
	  vuser3: vec3_t;
	  vuser4: vec3_t;
end;



type
  playermove_s = ^playermove_t;
  playermove_t = record
	  player_index        : integer;
	  server              : qboolean;
	  multiplayer         : qboolean;
	  time                : single;
	  frametime           : single;
	  _forward, right, up : vec3_t;
	  origin              : vec3_t;
	  angles              : vec3_t;
	  oldangles           : vec3_t;
	  velocity            : vec3_t;
	  movedir             : vec3_t;
	  basevelocity        : vec3_t;
	  view_ofs            : vec3_t;
	  flDuckTime          : single;
	  bInDuck             : qboolean;
	  flTimeStepSound     : integer;
	  iStepLeft           : integer;
	  flFallVelocity      : single;
	  punchangle          : vec3_t;
	  flSwimTime          : single;
	  flNextPrimaryAttack : single;
	  effects             : integer;
	  flags               : integer;
	  usehull             : integer;
	  gravity             : single;
	  friction            : single;
	  oldbuttons          : integer;
	  waterjumptime       : single;
	  dead                : qboolean;
	  deadflag            : integer;
	  spectator           : integer;
	   movetype           : integer;
	  onground            : integer;
	  waterlevel          : integer;
	  watertype           : integer;
	  oldwaterlevel       : integer;
	  sztexturename       : array[0..256] of char;
	  chtexturetype       : char;
	  maxspeed            : single;
	  clientmaxspeed      : single;
	  iuser1              : integer;
	  iuser2              : integer;
	  iuser3              : integer;
	  iuser4              : integer;
	  fuser1              : single;
	  fuser2              : single;
	  fuser3              : single;
	  fuser4              : single;
	  vuser1              : vec3_t;
	  vuser2              : vec3_t;
	  vuser3              : vec3_t;
	  vuser4              : vec3_t;
	  numphysent          : integer;
	  physents            : array[0..MAX_PHYSENTS] of physent_t;
	  nummoveent          : integer;
	  moveents            : array[0..MAX_MOVEENTS] of physent_t;
	  numvisent           : integer;
	  visents             : array[0..MAX_PHYSENTS] of physent_t;
	  cmd                 : usercmd_t;
	  numtouch            : integer;
	  touchindex          : array[0..MAX_PHYSENTS] of pmtrace_t;
	  physinfo            : array[0..MAX_PHYSINFO_STRING] of char;
	  movevars            : pointer; //movevars_t;
	  player_mins         : array[0..4] of vec3_t;
	  player_maxs          : array[0..4] of vec3_t;
	  PM_Info_ValueForKey   : function(const s, key: pchar):char;cdecl;
	  PM_Particle           : procedure(origin: vec3_t; color, live, zpos, zvel: integer);cdecl;
	  PM_TestPlayerPosition : function(pos: vec3_t; ptrace: pmtrace_s):integer;cdecl;
	  Con_NPrintf           : procedure(idx: integer; fmt: pchar);cdecl;
	  Con_DPrintf           : procedure(fmt: pchar);cdecl;
  	Con_Printf            : procedure(fmt: pchar);cdecl;
	  Sys_FloatTime         : function():double;cdecl;
	  PM_PointContents      : function(p: vec3_t; truecontents: pointer):integer;cdecl;
  	PM_TruePointContents  : function(p: vec3_t):integer;cdecl;
  	PM_HullPointContents  : function(hull: pointer; num: integer; p: vec3_t):integer;cdecl;
	  PM_PlayerTrace        : function(start, _end: vec3_t; traceFlags, ignore_pe: integer):pmtrace_t;cdecl;
	  PM_TraceLine          : function(start, _end: pSingle; flags, usehull, ignore_pe: integer):pmtrace_s;cdecl;
	  RandomLong            : function(lLow, lHigh: Cardinal):Cardinal;cdecl;
	  RandomFloat           : function(lLow, lHigh: Single):Single;cdecl;
	  PM_GetModelType       : function(_mod: model_s): integer;cdecl;
	  PM_GetModelBounds     : procedure(_mod: model_s; mins, maxs: vec3_t);cdecl;
  	PM_HullForBsp         : procedure(pe: pointer; offset: vec3_t);cdecl;
  	PM_TraceModel         : function(pEnt: pointer; start, _end: vec3_t; trace: pmtrace_s):Single;cdecl;
	  COM_FileSize          : function(filename: pchar):integer;cdecl;
	  COM_LoadFile          : function(path: pchar; usehunk: integer; pLength: pinteger):byte;cdecl;
	  COM_FreeFile          : procedure(buffer: pointer);cdecl;
  	memfgets              : function(pMemFile: pbyte; fileSize: integer; pFilePos: pinteger; pBuffer: pchar; bufferSize: integer):char;cdecl;
	  runfuncs: qboolean;
	  PM_PlaySound          : procedure(channel: integer; const sample: pchar; volume, attenuation: single; fFlags, pitch: integer);cdecl;
	  PM_TraceTexture       : function(ground: integer; vstart, vend: vec3_t):char;cdecl;
	  PM_PlaybackEventFull  : procedure(flags, clientindex: integer; eventindex: single; delay: single; origin, angles: psingle; fparam1, fparam2: single; iparam1, iparam2, bparam1, bparam2: integer);cdecl;
end;


type
  ref_params_s = ^ref_params_t;
  ref_params_t = record
    vieworg        : vec3_t;
    viewangles     : vec3_t;
    _forward       : vec3_t;
    right          : vec3_t;
    up             : vec3_t;
    frametime      : single;
    time           : single;
    intermission   : longint;
    paused         : longint;
    spectator      : longint;
    onground       : longint;
    waterlevel     : longint;
    simvel         : vec3_t;
    simorg         : vec3_t;
    viewheight     : vec3_t;
    idealpitch     : single;
    cl_viewangles  : vec3_t;
    health         : longint;
    crosshairangle : vec3_t;
    viewsize       : single;
    punchangle     : vec3_t;
    maxclients     : longint;
    viewentity     : longint;
    playernum      : longint;
    max_entities   : longint;
    demoplayback   : longint;
    hardware       : longint;
    smoothing      : longint;
    cmd            : usercmd_s;
    movevars       : Pointer;
    viewport       : array[0..3] of longint;
    nextView       : longint;
    onlyClientDraw : longint;
  end;

type
  tempent_s = ^tempent_t;
  tempent_t = packed record
    flags     : Integer;
    die       : Single;
    frameMax  : Single;
    x,y,z     : Single;
    fadeSpeed : Single;
    bounceFac : Single;
    hitSound  : Integer;
    hitcallb  : Pointer;
    callback  : Pointer;
    next      : tempent_s;
    prority   : Integer;
    clientInde: ShortInt;
    tentOff   : vec3_t;
    entity    : cl_entity_s;
end;

type
  studiohdr_s = ^studiohdr_t;
  studiohdr_t = record
    id, version         : Integer;
    name                : Array[0..63] of Char;
    length              : Integer;
    eyeposition, min    : vec3_t;
    max, bbmin, bbmax   : vec3_t;
    flags, numbones     : Integer;
    boneindex           : Integer;
    numbonecontrollers  : Integer;
    bonecontrollerindex : Integer;
    numhitboxes         : Integer;
    hitboxindex         : Integer;
    numseq              : Integer;
    seqindex            : Integer;
    numseqgroups        : Integer;
    seqgroupindex       : Integer;
    numtextures         : Integer;
    textureindex        : Integer;
    texturedataindex    : Integer;
    numskinref          : Integer;
    numskinfamilies     : Integer;
    skinindex           : Integer;
    numbodyparts        : Integer;
    bodypartindex       : Integer;
    numattachments      : Integer;
    attachmentindex     : Integer;
    soundtable          : Integer;
    soundindex          : Integer;
    soundgroups         : Integer;
    soundgroupindex     : Integer;
    numtransitions      : Integer;
    transitionindex     : Integer;
  end;

type
  mstudiobbox_s = ^mstudiobbox_t;
  mstudiobbox_t = record
    bone, group   : Integer;
    bbmin, bbmax  : vec3_t;
  end;


type
  BoneMatrix_s = ^BoneMatrix_t;
  BoneMatrix_t = Array[0..127] of Array[0..2] of Array[0..3] of Single;
  //BoneMatrix_t = Array[0..127] of Array[0..2] of Array[0..4] of Single;
type
  r_studio_interface_ss = ^r_studio_interface_s;
  r_studio_interface_s = ^r_studio_interface_t;
  r_studio_interface_t = record
    version         : Integer;
    StudioDrawModel : function(flags: Integer):Integer;cdecl;
    StudioDrawPlayer: function(flags: Integer; pplayer: entity_state_s): Integer;cdecl;
  end;

type
  weapon_data_s = record
          m_iId : longint;
          m_iClip : longint;
          m_flNextPrimaryAttack : Single;
          m_flNextSecondaryAttack : Single;
          m_flTimeWeaponIdle : Single;
          m_fInReload : longint;
          m_fInSpecialReload : longint;
          m_flNextReload : Single;
          m_flPumpTime : Single;
          m_fReloadTime : Single;
          m_fAimedDamage : Single;
          m_fNextAimBonus : Single;
          m_fInZoom : longint;
          m_iWeaponState : longint;
          iuser1 : longint;
          iuser2 : longint;
          iuser3 : longint;
          iuser4 : longint;
          fuser1 : Single;
          fuser2 : Single;
          fuser3 : Single;
          fuser4 : Single;
       end;
     weapon_data_t = weapon_data_s;

type
  clientdata_s = record
          origin : vec3_t;
          velocity : vec3_t;
          viewmodel : longint;
          punchangle : vec3_t;
          flags : longint;
          waterlevel : longint;
          watertype : longint;
          view_ofs : vec3_t;
          health : Single;
          bInDuck : longint;
          weapons : longint;
          flTimeStepSound : longint;
          flDuckTime : longint;
          flSwimTime : longint;
          waterjumptime : longint;
          maxspeed : Single;
          fov : Single;
          weaponanim : longint;
          m_iId : longint;
          ammo_shells : longint;
          ammo_nails : longint;
          ammo_cells : longint;
          ammo_rockets : longint;
          m_flNextAttack : Single;
          tfstate : longint;
          pushmsec : longint;
          deadflag : longint;
          physinfo : array[0..(MAX_PHYSINFO_STRING)-1] of char;
          iuser1 : longint;
          iuser2 : longint;
          iuser3 : longint;
          iuser4 : longint;
          fuser1 : Single;
          fuser2 : Single;
          fuser3 : Single;
          fuser4 : Single;
          vuser1 : vec3_t;
          vuser2 : vec3_t;
          vuser3 : vec3_t;
          vuser4 : vec3_t;
       end;
     clientdata_t = clientdata_s;

type
  local_state_s = ^local_state_t;
  local_state_t = record
    playerstate:   entity_state_t;
    client     :   clientdata_t;
    weapondata :   Array[0..31] of weapon_data_t;
  end;


//***************************************************************************//
//                                                                           //
//                                ProcDefs                                   //
//                                                                           //
//***************************************************************************//
type
  pfnUserMsgHook = function(const pszName: Pchar; iSize: Integer; pbuf: Pointer): Integer;cdecl;
  pfnEvent   =  procedure(args: event_args_s);cdecl;
  PTTempEntPlaySoundFunc = ^TTempEntPlaySoundFunc;
  TTempEntPlaySoundFunc = procedure(pTemp: tempent_s; damp: Single);cdecl;
//***************************************************************************//
//                                                                           //
//                                  Apis                                     //
//                                                                           //
//***************************************************************************//
type
  event_api_s = ^event_api_t;
  event_api_t = record
    version                   : longint;
    EV_PlaySound              : procedure (ent:longint; origin:Psingle; channel:longint; sample:Pchar; volume:single;attenuation:single; fFlags:longint; pitch:longint);cdecl;
    EV_StopSound              : procedure (ent:longint; channel:longint; sample:Pchar);cdecl;
    EV_FindModelIndex         : function (pmodel:Pchar):longint;cdecl;
    EV_IsLocal                : function (playernum:longint):longint;cdecl;
    EV_LocalPlayerDucking     : function :longint;cdecl;
    EV_LocalPlayerViewheight  : procedure (_para1:Psingle);cdecl;
    EV_LocalPlayerBounds      : procedure (hull:longint; mins:Psingle; maxs:Psingle);cdecl;
    EV_IndexFromTrace         : function (pTrace:pmtrace_s):longint;cdecl;
    EV_GetPhysent             : function (idx:longint):Pointer;cdecl;
    EV_SetUpPlayerPrediction  : procedure (dopred:longint; bIncludeLocalClient:longint);cdecl;
    EV_PushPMStates           : procedure ;cdecl;
    EV_PopPMStates            : procedure ;cdecl;
    EV_SetSolidPlayers        : procedure (playernum:longint);cdecl;
    EV_SetTraceHull           : procedure (hull:longint);cdecl;
    EV_PlayerTrace            : procedure (start:Psingle; _end:Psingle; traceFlags:longint; ignore_pe:longint; tr:pmtrace_s);cdecl;
    EV_WeaponAnimation        : procedure (sequence:longint; body:longint);cdecl;
    EV_PrecacheEvent          : function (_type:longint; psz:Pchar):word;cdecl;
    EV_PlaybackEvent          : procedure (flags:longint; pInvoker:Pointer; eventindex:word; delay:single; origin:Psingle;angles:Psingle; fparam1:single; fparam2:single; iparam1:longint; iparam2:longint;bparam1:longint; bparam2:longint);cdecl;
    EV_TraceTexture           : function (ground:longint; vstart:Psingle; vend:Psingle):Pchar;cdecl;
    EV_StopAllSounds          : procedure (entnum:longint; entchannel:longint);cdecl;
    EV_KillEvents             : procedure (entnum:longint; eventname:Pchar);cdecl;
  end;

type
  efx_api_s = ^efx_api_t;
  efx_api_t = record
    R_AllocParticle                 : function (callback:pointer):Pointer;cdecl;
    R_BlobExplosion                 : procedure (org:Psingle);cdecl;
    R_Blood                         : procedure (org:Psingle; dir:Psingle; pcolor:longint; speed:longint);cdecl;
    R_BloodSprite                   : procedure (org:Psingle; colorindex:longint; modelIndex:longint; modelIndex2:longint; size:single);cdecl;
    R_BloodStream                   : procedure (org:Psingle; dir:Psingle; pcolor:longint; speed:longint);cdecl;
    R_BreakModel                    : procedure (pos:Psingle; size:Psingle; dir:Psingle; random:single; life:single;count:longint; modelIndex:longint; flags:char);cdecl;
    R_Bubbles                       : procedure (mins:Psingle; maxs:Psingle; height:single; modelIndex:longint; count:longint;speed:single);cdecl;
    R_BubbleTrail                   : procedure (start:Psingle; __end:Psingle; height:single; modelIndex:longint; count:longint;speed:single);cdecl;
    R_BulletImpactParticles         : procedure (pos:Psingle);cdecl;
    R_EntityParticles               : procedure (ent:cl_entity_s);cdecl;
    R_Explosion                     : procedure (pos:Psingle; model:longint; scale:single; framerate:single; flags:longint);cdecl;
    R_FizzEffect                    : procedure (pent:cl_entity_s; modelIndex:longint; density:longint);cdecl;
    R_FireField                     : procedure (org:Psingle; radius:longint; modelIndex:longint; count:longint; flags:longint;life:single);cdecl;
    R_FlickerParticles              : procedure (org:Psingle);cdecl;
    R_FunnelSprite                  : procedure (org:Psingle; modelIndex:longint; reverse:longint);cdecl;
    R_Implosion                     : procedure (__end:Psingle; radius:single; count:longint; life:single);cdecl;
    R_LargeFunnel                   : procedure (org:Psingle; reverse:longint);cdecl;
    R_LavaSplash                    : procedure (org:Psingle);cdecl;
    R_MultiGunshot                  : procedure (org:Psingle; dir:Psingle; noise:Psingle; count:longint; decalCount:longint;decalIndices:Plongint);cdecl;
    R_MuzzleFlash                   : procedure (pos1:Psingle; _type:longint);cdecl;
    R_ParticleBox                   : procedure (mins:Psingle; maxs:Psingle; r:byte; g:byte; b:byte;life:single);cdecl;
    R_ParticleBurst                 : procedure (pos:Psingle; size:longint; color:longint; life:single);cdecl;
    R_ParticleExplosion             : procedure (org:Psingle);cdecl;
    R_ParticleExplosion2            : procedure (org:Psingle; colorStart:longint; colorLength:longint);cdecl;
    R_ParticleLine                  : procedure (start:Psingle; _end:Psingle; r:byte; g:byte; b:byte;life:single);cdecl;
    R_PlayerSprites                 : procedure (client:longint; modelIndex:longint; count:longint; size:longint);cdecl;
    R_Projectile                    : procedure (origin:Psingle; velocity:Psingle; modelIndex:longint; life:longint; owner:longint;hitcallback:Pointer);cdecl;
    R_RicochetSound                 : procedure (pos:Psingle);cdecl;
    R_RicochetSprite                : procedure (pos:Psingle; pmodel:model_s; duration:single; scale:single);cdecl;
    R_RocketFlare                   : procedure (pos:Psingle);cdecl;
    R_RocketTrail                   : procedure (start:Psingle; _end:Psingle; _type:longint);cdecl;
    R_RunParticleEffect             : procedure (org:Psingle; dir:Psingle; color:longint; count:longint);cdecl;
    R_ShowLine                      : procedure (start:Psingle; _end:Psingle);cdecl;
    R_SparkEffect                   : procedure (pos:Psingle; count:longint; velocityMin:longint; velocityMax:longint);cdecl;
    R_SparkShower                   : procedure (pos:Psingle);cdecl;
    R_SparkStreaks                  : procedure (pos:Psingle; count:longint; velocityMin:longint; velocityMax:longint);cdecl;
    R_Spray                         : procedure (pos:Psingle; dir:Psingle; modelIndex:longint; count:longint; speed:longint; spread:longint; rendermode:longint);cdecl;
    R_Sprite_Explode                : procedure (pTemp:Pointer{PTEMPENTITY}; scale:single; flags:longint);cdecl;
    R_Sprite_Smoke                  : procedure (pTemp:Pointer{PTEMPENTITY}; scale:single);cdecl;
    R_Sprite_Spray                  : procedure (pos:Psingle; dir:Psingle; modelIndex:longint; count:longint; speed:longint;iRand:longint);cdecl;
    R_Sprite_Trail                  : procedure (_type:longint; start:Psingle; _end:Psingle; modelIndex:longint; count:longint;life:single; size:single; amplitude:single; renderamt:longint; speed:single);cdecl;
    R_Sprite_WallPuff               : procedure (pTemp:Pointer{PTEMPENTITY}; scale:single);cdecl;
    R_StreakSplash                  : procedure (pos:Psingle; dir:Psingle; color:longint; count:longint; speed:single;velocityMin:longint; velocityMax:longint);cdecl;
    R_TracerEffect                  : procedure (start:Psingle; _end:Psingle);cdecl;
    R_UserTracerParticle            : procedure (org:Psingle; vel:Psingle; life:single; colorIndex:longint; length:single;deathcontext:byte; deathfunc:Pointer);cdecl;
    R_TracerParticles               : function (org:Psingle; vel:Psingle; life:single):Pointer;cdecl;
    R_TeleportSplash                : procedure (org:Psingle);cdecl;
    R_TempSphereModel               : procedure (pos:Psingle; speed:single; life:single; count:longint; modelIndex:longint); cdecl;
    R_TempModel                     : function (pos:Psingle; dir:Psingle; angles:Psingle; life:single; modelIndex:longint;soundtype:longint):Pointer;cdecl;//^TEMPENTITY;cdecl;
    R_DefaultSprite                 : function (pos:Psingle; spriteIndex:longint; framerate:single):Pointer;cdecl;//^TEMPENTITY;cdecl;
    R_TempSprite                    : function (pos:Psingle; dir:Psingle; scale:single; modelIndex:longint; rendermode:longint;renderfx:longint; a:single; life:single; flags:longint):Pointer;cdecl;//^TEMPENTITY;
    Draw_DecalIndex                 : function (id:longint):longint;cdecl;
    Draw_DecalIndexFromName         : function (name:Pchar):longint;  cdecl;
    R_DecalShoot                    : procedure (textureIndex:longint; entity:longint; modelIndex:longint; position:Psingle; flags:longint); cdecl;
    R_AttachTentToPlayer            : procedure (client:longint; modelIndex:longint; zoffset:single; life:single);  cdecl;
    R_KillAttachedTents             : procedure (client:longint);cdecl;
    R_BeamCirclePoints              : function (_type:longint; start:Psingle; _end:Psingle; modelIndex:longint; life:single;width:single; amplitude:single; brightness:single; speed:single; startFrame:longint;framerate:single; r:single; g:single; b:single):Pointer;cdecl;//^BEAM;
    R_BeamEntPoint                  : function (startEnt:longint; _end:Psingle; modelIndex:longint; life:single; width:single;amplitude:single; brightness:single; speed:single; startFrame:longint; framerate:single;r:single; g:single; b:single):Pointer;cdecl;//^BEAM;
    R_BeamEnts                      : function (startEnt:longint; endEnt:longint; modelIndex:longint; life:single; width:single;amplitude:single; brightness:single; speed:single; startFrame:longint; framerate:single;r:single; g:single; b:single):Pointer;cdecl;//^BEAM;
    R_BeamFollow                    : function (startEnt:longint; modelIndex:longint; life:single; width:single; r:single;g:single; b:single; brightness:single):Pointer;cdecl;//^BEAM;
    R_BeamKill                      : procedure (deadEntity:longint);cdecl;
    R_BeamLightning                 : function (start:Psingle; _end:Psingle; modelIndex:longint; life:single; width:single;amplitude:single; brightness:single; speed:single):Pointer;cdecl;//^BEAM;
    R_BeamPoints                    : function (start:Psingle; _end:Psingle; modelIndex:longint; life:single; width:single;amplitude:single; brightness:single; speed:single; startFrame:longint; framerate:single;r:single; g:single; b:single):Pointer;cdecl;//^BEAM;
    R_BeamRing                      : function (startEnt:longint; endEnt:longint; modelIndex:longint; life:single; width:single;amplitude:single; brightness:single; speed:single; startFrame:longint; framerate:single;r:single; g:single; b:single):Pointer;cdecl;//^BEAM;
    CL_AllocDlight                  : function (key:longint):Pointer;cdecl;//^dlight_t;
    CL_AllocElight                  : function (key:longint):Pointer;cdecl;//^dlight_t;
    CL_TempEntAlloc                 : function (org:Psingle; model:model_s):Pointer;cdecl;//^TEMPENTITY;
    CL_TempEntAllocNoModel          : function (org:Psingle):Pointer;cdecl;//^TEMPENTITY;
    CL_TempEntAllocHigh             : function (org:Psingle; model:model_s):Pointer;cdecl;//^TEMPENTITY;
    CL_TentEntAllocCustom           : function (origin:Psingle; model:model_s; high:longint; callback:Pointer):Pointer;cdecl;//^TEMPENTITY;
    R_GetPackedColor                : procedure (_packed:Psmallint; color:smallint);cdecl;
    R_LookupColor                   : function (r:byte; g:byte; b:byte):smallint;  cdecl;
    R_DecalRemoveAll                : procedure (textureIndex:longint);cdecl;
end;

type
  cl_enginefuncs_s = ^cl_enginefuncs_t;
  cl_enginefuncs_t = record
    pfnSprLoad                          : function  (const szPicName: Pchar): HSPRITE;cdecl;
    pfnSPR_Frames                       : function  (hPic: HSPRITE): Integer;cdecl;
    pfnSPR_Height                       : function  (hPic: HSPRITE; frame: Integer):Integer;cdecl;
    pfnSPR_Width                        : function  (hPic: HSPRITE; frame: Integer):Integer;cdecl;
    pfnSPR_Set                          : procedure (hPic: HSPRITE; r,g,b: Integer);cdecl;
    pfnSPR_Draw                         : procedure (frame,x,y: Integer; prc :rect_s);cdecl;
    pfnSPR_DrawHoles                    : procedure (frame,x,y: Integer; prc :rect_s);cdecl;
    pfnSPR_DrawAdditive                 : procedure (frame,x,y: Integer; prc :rect_s);cdecl;
    pfnSPR_EnableScissor                : procedure (x,y,width,height: Integer);cdecl;
    pfnSPR_DisableScissor               : procedure ;cdecl;
{10}pfnSPR_GetList                      : function  (psz: Pchar; piCount: PInteger):client_sprite_s;cdecl; //client_sprite_s?
    pfnFillRGBA                         : procedure (x,y,width,height,r,g,b,a: Integer);cdecl;
    pfnGetScreenInfo                    : function  (pscrinfo: SCREENINFO_s): Integer;cdecl;
    pfnSetCrosshair                     : procedure (hspr: HSPRITE; rc :wrect_t; r,g,b: Integer);cdecl;
    pfnRegisterVariable                 : function  (szName, szValue: PChar; flags: Integer):cvar_s;cdecl; //cvar_s? unsure
    pfnGetCvarFloat                     : function  (szName: Pchar): single;cdecl;
    pfnGetCvarString                    : function  (szName: Pchar): PChar;cdecl;
    pfnAddCommand                       : function  (cmd_name: PChar; _function: Pointer): Integer;cdecl;
    pfnHookUserMsg                      : function  (szMsgName: Pchar; pfn: pfnUserMsgHook): Integer;cdecl;
    pfnServerCmd                        : function  (szCmdString: Pchar):Integer;cdecl;
{20}pfnClientCmd                        : function  (szCmdString: Pchar):Integer;cdecl;
    pfnGetPlayerInfo                    : procedure (ent_num: Integer; pinfo: hud_player_info_s);cdecl;
    pfnPlaySoundByName                  : procedure (szSound:Pchar; volume: single);cdecl;
    pfnPlaySoundByIndex                 : procedure (iSound: LongInt; volume: single);cdecl;
    pfnAngleVectors                     : procedure (vecAngles: Psingle; _forward: Pointer; right: Pointer; up: Pointer);cdecl;
    pfnTextMessageGet                   : function  (pName: Pchar):client_textmessage_s;cdecl;
    pfnDrawCharacter                    : function (x:longint; y:longint; number:longint; r:longint; g:longint;b:longint):longint;cdecl;
    pfnDrawConsoleString                : function (x:longint; y:longint; _string:Pchar):longint;cdecl;
    pfnDrawSetTextColor                 : procedure (r:single; g:single; b:single);cdecl;
    pfnDrawConsoleStringLen             : procedure (_string:Pchar; length:Plongint; height:Plongint);cdecl;
{30}pfnConsolePrint                     : procedure (_string:Pchar);cdecl;
    pfnCenterPrint                      : procedure (_string:Pchar);cdecl;
    GetWindowCenterX                    : function :longint;cdecl;
    GetWindowCenterY                    : function :longint;cdecl;
    GetViewAngles                       : procedure (_param1:vec3_s);cdecl;
    SetViewAngles                       : procedure (_param1:vec3_s);cdecl;
    GetMaxClients                       : function :longint;cdecl;
    Cvar_SetValue                       : procedure (cvar:Pchar; value:single);cdecl;
    Cmd_Argc                            : function :longint;cdecl;
    Cmd_Argv                            : function (arg:longint): Pchar; cdecl;
{40}Con_Printf                          : procedure (fmt:Pchar; args:array of const);cdecl;
    Con_DPrintf                         : procedure (fmt:Pchar; args:array of const);cdecl;
    Con_NPrintf                         : procedure (pos:longint; fmt:Pchar; args:array of const);cdecl;
    Con_NXPrintf                        : procedure (info: con_nprint_s; fmt:Pchar; args:array of const);cdecl;
    PhysInfo_ValueForKey                : function (key:Pchar):Pchar;cdecl;
    ServerInfo_ValueForKey              : function (key:Pchar):Pchar;cdecl;
    GetClientMaxspeed                   : function :single;cdecl;
    CheckParm                           : function (parm:Pchar; ppnext:PPchar):longint;cdecl;
    Key_Event                           : procedure (key:longint; down:longint);cdecl;
    GetMousePosition                    : procedure (mx:Plongint; my:Plongint);cdecl;
{50}IsNoClipping                        : function :longint;cdecl;
    GetLocalPlayer                      : function :cl_entity_s;cdecl;
    GetViewModel                        : function :cl_entity_s;cdecl;
    GetEntityByIndex                    : function (idx:longint):cl_entity_s;cdecl;
    GetClientTime                       : function :single;cdecl;
    V_CalcShake                         : procedure ;cdecl;
    V_ApplyShake                        : procedure (origin:Psingle; angles:Psingle; factor:single);cdecl;
    PM_PointContents                    : function (point:Psingle; truecontents:Plongint):longint;cdecl;
    PM_WaterEntity                      : function (p:Psingle):longint;cdecl;
    PM_TraceLine                        : function (start:Psingle; _end:Psingle; flags:longint; usehull:longint; ignore_pe:longint):pmtrace_s;cdecl;
    CL_LoadModel                        : function (modelname:Pchar; index:Plongint):Pointer;cdecl; //model_s
    CL_CreateVisibleEntity              : function (_type:longint; ent:cl_entity_s):longint;cdecl;
    GetSpritePointer                    : function (hSprite:HSPRITE):Pointer;cdecl; //model_s
    pfnPlaySoundByNameAtLocation        : procedure (szSound:Pchar; volume:single; origin:Psingle);cdecl;
    pfnPrecacheEvent                    : function (_type:longint; psz:Pchar):word;cdecl;
    pfnPlaybackEvent                    : procedure (flags:longint; pInvoker:Pointer{edict_s}; eventindex:word; delay:single; origin:Psingle;angles:Psingle; fparam1:single; fparam2:single; iparam1:longint; iparam2:longint;bparam1:longint; bparam2:longint);cdecl;
    pfnWeaponAnim                       : procedure (iAnim:longint; body:longint);cdecl;
    pfnRandomFloat                      : function (flLow:single; flHigh:single):single;cdecl;
    pfnRandomLong                       : function (lLow:longint; lHigh:longint):longint;cdecl;
    pfnHookEvent                        : procedure (name:Pchar; pfnEvent:Pointer);cdecl;
    Con_IsVisible                       : function :longint;cdecl;
    pfnGetGameDirectory                 : function :Pchar;cdecl;
    pfnGetCvarPointer                   : function (szName:Pchar):cvar_s;cdecl;
    Key_LookupBinding                   : function (pBinding:Pchar):Pchar;cdecl;
    pfnGetLevelName                     : function :Pchar;cdecl;
    pfnGetScreenFade                    : procedure (fade:screenfade_s);cdecl;
    pfnSetScreenFade                    : procedure (fade:screenfade_s);cdecl;
    VGui_GetPanel                       : function :pointer;cdecl;
    VGui_ViewportPaintBackground        : procedure (extents: LongIntArr4);cdecl;
    COM_LoadFile                        : function (path:Pchar; usehunk:longint; pLength:Plongint):Pbyte;cdecl;
    COM_ParseFile                       : function (data:Pchar; token:Pchar):Pchar;cdecl;
    COM_FreeFile                        : procedure (buffer:pointer);cdecl;
    pTriAPI                             : triangleapi_s; //^triangleapi_s;
    pEfxAPI                             : efx_api_s ; 
    pEventAPI                           : event_api_s;
    pDemoAPI                            : Pointer; //^demo_api_s;
    pNetAPI                             : Pointer; //^net_api_s;
    pVoiceTweak                         : Pointer; //^IVoiceTweak_s;
    IsSpectateOnly                      : function :longint;cdecl;
    LoadMapSprite                       : function (filename:Pchar):Pointer;cdecl; //model_s;
    COM_AddAppDirectoryToSearchPath     : procedure (pszBaseDir:Pchar; appName:Pchar);cdecl;
    COM_ExpandFilename                  : function (fileName:Pchar; nameOutBuffer:Pchar; nameOutBufferSize:longint):longint;cdecl;
    PlayerInfo_ValueForKey              : function (playerNum:longint; key:Pchar):Pchar;cdecl;
    PlayerInfo_SetValueForKey           : procedure (key:Pchar; value:Pchar);cdecl;
    GetPlayerUniqueID                   : function (iPlayer:longint; playerID: CharArr16):qboolean;cdecl;
    GetTrackerIDForPlayer               : function (playerSlot:longint):longint;cdecl;
    GetPlayerForTrackerID               : function (trackerID:longint):longint;cdecl;
    pfnServerCmdUnreliable              : function (szCmdString:Pchar):longint;cdecl;
    pfnGetMousePos                      : procedure (ppt:PPOINT);cdecl;
    pfnSetMousePos                      : procedure (x:longint; y:longint);cdecl;
    pfnSetMouseEnable                   : procedure (fEnable:qboolean);cdecl;
    pfnGetCvarList                      : function :cvar_s;cdecl;
    pfnGetCmdList                       : function : Pointer; cdecl;//cmd_s;
    pfnGetCvarName                      : function (cvar:cvar_s):Pchar;cdecl;
    pfnGetCmdName                       : function (cmd:Pointer {cmd_s}):Pchar;cdecl;
    pfnGetServerTime                    : function :single;cdecl;
    pfnGetGravity                       : function :single;cdecl;
    pfnPrecacheSprite                   : function (spr:HSPRITE):Pointer;cdecl; //model_s;
    OverrideLightmap                    : procedure (override:longint);cdecl;
    SetLightmapColor                    : procedure (r:single; g:single; b:single);cdecl;
    SetLightmapDarkness                 : procedure (dark:single);cdecl;
    pfnGetSequenceByName                : function (flags:longint; seq:Pchar):longint;cdecl;
    pfnSPR_DrawGeneric                  : procedure (frame:longint; x:longint; y:longint; prc:rect_s; blendsrc:longint;blenddst:longint; unknown1:longint; unknown2:longint);cdecl;
    pfnLoadSentence                     : procedure ;cdecl;
    pfnDrawLocalizedHudString           : function (x:longint; y:longint; str:Pchar; r:longint; g:longint;b:longint):longint;cdecl;
    pfnDrawLocalizedConsoleString       : function (x:longint; y:longint; str:Pchar):longint;cdecl;
    LocalPlayerInfo_ValueForKey         : function (key:Pchar):Pchar;cdecl;
    pfnDrawText_0                       : procedure (x:longint; y:longint; text:Pchar; font:dword);cdecl;
    pfnDrawUnicodeCharacter             : function (x:longint; y:longint; number:smallint; r:longint; g:longint;b:longint; hfont:dword):longint;cdecl;
    pfnCheckSoundFile                   : function (path:Pchar):longint;cdecl;
    GetCareerGameInterface              : function   :pointer;cdecl;
    pfnCvar_Set                         : procedure (cvar:Pchar; value:Pchar);cdecl;
    IsSinglePlayer                      : function :longint;cdecl;
    pfnPlaySound                        : procedure (sound:Pchar; vol:single; pitch:single);cdecl;
    pfnPlayMp3                          : procedure (mp3:Pchar; flags:longint);cdecl;
    Sys_FloatTime                       : function :single;cdecl;
    pfnSetArray                         : procedure (_array:Plongint; size:longint);cdecl;
    pfnSetClearArray                    : procedure (_array:Plongint; size:longint);cdecl;
    pfnClearArray                       : procedure ;cdecl;
    pfnPlaySound2                       : procedure (sound:Pchar; vol:single; pitch:single);cdecl;
    pfnTintRGBA                         : procedure (x:longint; y:longint; width:longint; height:longint; r:longint; g:longint; b:longint; a:longint);cdecl;
  end;

type
  engine_studio_api_s = ^engine_studio_api_t;
  engine_studio_api_t = record
  Mem_Calloc              : function (number:longint; size:int64):pointer;cdecl;
  Cache_Check             : function (c:Pointer):pointer;cdecl;
  LoadCacheFile           : procedure (path:Pchar; cu:pointer);cdecl;
  Mod_ForName             : function (name:Pchar; crash_if_missing:longint):model_s;cdecl;
  Mod_Extradata           : function (_mod:model_s):studiohdr_s;cdecl;
  GetModelByIndex         : function (index:longint):model_s;cdecl;
  GetCurrentEntity        : function :cl_entity_s;cdecl;
  PlayerInfo              : function (index:longint):pointer;cdecl;
  GetPlayerState          : function (index:longint):entity_state_s;cdecl;
  GetViewEntity           : function :cl_entity_s;cdecl;
  GetTimes                : procedure (framecount:Plongint; current:Psingle; old:Psingle);cdecl;
  GetCvar                 : function (name:Pchar):cvar_s;cdecl;
  GetViewInfo             : procedure (origin:Psingle; upv:Psingle; rightv:Psingle; vpnv:Psingle);cdecl;
  GetChromeSprite         : function :model_s;cdecl;
  GetModelCounters        : procedure (s:Plongint; a:Plongint);cdecl;
  GetAliasScale           : procedure (x:Psingle; y:Psingle);cdecl;
  StudioGetBoneTransform  : function :Psingle;cdecl;
  StudioGetLightTransform : function :Psingle;cdecl;
  StudioGetAliasTransform : function :Psingle;cdecl;
  StudioGetRotationMatrix : function :Psingle;cdecl;
  StudioSetupModel        : procedure (bodypart:longint; ppbodypart:Ppointer; ppsubmodel:Ppointer);cdecl;
  StudioCheckBBox         : function :longint;cdecl;
  StudioDynamicLight      : procedure (ent:cl_entity_s; plight:pointer);cdecl;
  StudioEntityLight       : procedure (plight:pointer);cdecl;
  StudioSetupLighting     : procedure (plighting:pointer);cdecl;
  StudioDrawPoints        : procedure ;cdecl;
  StudioDrawHulls         : procedure ;cdecl;
  StudioDrawAbsBBox       : procedure ;cdecl;
  StudioDrawBones         : procedure ;cdecl;
  StudioSetupSkin         : procedure (ptexturehdr:pointer; index:longint);cdecl;
  StudioSetRemapColors    : procedure (top:longint; bottom:longint);cdecl;
  SetupPlayerModel        : function (index:longint):model_s;cdecl;
  StudioClientEvents      : procedure ;cdecl;
  GetForceFaceFlags       : function :longint;cdecl;
  SetForceFaceFlags       : procedure (flags:longint);cdecl;
  StudioSetHeader         : procedure (header:pointer);cdecl;
  SetRenderModel          : procedure (model:model_s);cdecl;
  SetupRenderer           : procedure (rendermode:longint);cdecl;
  RestoreRenderer         : procedure ;cdecl;
  SetChromeOrigin         : procedure ;cdecl;
  IsHardware              : function :longint;cdecl;
  GL_StudioDrawShadow     : procedure ;cdecl;
  GL_SetRenderMode        : procedure (mode:longint);cdecl;
end;

type
  ExportTable_s = ^ExportTable_t;
  ExportTable_t = record
    Initialize                    : function (_para1:cl_enginefuncs_s; _para2:longint):longint;cdecl;
    HUD_Init                      : function :longint;cdecl;
    HUD_VidInit                   : function :longint;cdecl;
    HUD_Redraw                    : function (flTime:single; intermission:longint):longint;cdecl;
    HUD_UpdateClientData          : function (_para1: Pointer; _para2:single):longint;cdecl;
    HUD_Reset                     : function :longint;cdecl;
    HUD_PlayerMove                : procedure(a :playermove_s; b:longint);cdecl;
    HUD_PlayerMoveInit            : procedure (_para1:Pointer);cdecl;
    HUD_PlayerMoveTexture         : function (_para1:Pchar):char;cdecl;
    IN_ActivateMouse              : procedure ;cdecl;
    IN_DeactivateMouse            : procedure ;           cdecl;
    IN_MouseEvent                 : procedure (mstate:longint);cdecl;
    IN_ClearStates                : procedure ;cdecl;
    IN_Accumulate                 : procedure ;cdecl;
    CL_CreateMove                 : procedure (_para1:single; _para2:usercmd_s; _para3:longint);cdecl;
    CL_IsThirdPerson              : function :longint;cdecl;
    CL_CameraOffset               : procedure (_para1:Psingle);cdecl;
    KB_Find                       : function (_para1:Pchar):Pointer;cdecl;
    CAM_Think                     : procedure ;cdecl;
    V_CalcRefdef                  : procedure (pparams:ref_params_s);cdecl;
    HUD_AddEntity                 : function (_para1:longint; _para2:cl_entity_s; _para3:Pchar):longint;cdecl;
    HUD_CreateEntities            : procedure ;cdecl;
    HUD_DrawNormalTriangles       : procedure ;cdecl;
    HUD_DrawTransparentTriangles  : procedure ;cdecl;
    HUD_StudioEvent               : procedure (_para1:pointer; _para2:cl_entity_s);cdecl;
    HUD_PostRunCmd                : procedure (from: local_state_s; _to: local_state_s; cmd:usercmd_s; runfuncs:longint; time:single; random_seed:dword);cdecl;
    HUD_Shutdown                  : procedure ;cdecl;
    HUD_TxferLocalOverrides       : procedure (_para1:Pointer; _para2:Pointer);cdecl;
    HUD_ProcessPlayerState        : procedure (_para1:Pointer; _para2:Pointer);cdecl;
    HUD_TxferPredictionData       : procedure (_para1:Pointer; _para2:Pointer; _para3:Pointer; _para4:Pointer; _para5:Pointer;_para6:Pointer);cdecl;
    Demo_ReadBuffer               : procedure (_para1:longint; _para2:Pbyte);cdecl;
    HUD_ConnectionlessPacket      : function (_para1:Pointer; _para2:Pchar; _para3:Pchar; _para4:Plongint):longint;cdecl;
    HUD_GetHullBounds             : function (_para1:longint; _para2:Psingle; _para3:Psingle):longint;cdecl;
    HUD_Frame                     : procedure (time:single);cdecl;
    HUD_Key_Event                 : function (_para1:longint; _para2:longint; _para3:Pchar):longint;cdecl;
    HUD_TempEntUpdate             : procedure (pframetime: Psingle; pclient_time:Psingle; pcl_gravity:single; _para4:Pointer; _para5:Pointer;Callback_AddVisibleEntity:Pointer; Callback_TempEntPlaySound:PTTempEntPlaySoundFunc);cdecl;
    HUD_GetUserEntity             : function (index:longint):cl_entity_s;cdecl;
    HUD_VoiceStatus               : function (entindex:longint; bTalking:qboolean):longint;cdecl;
    HUD_DirectorEvent             : function (_para1:byte; _para2:dword; _para3:dword; _para4:dword):longint;cdecl;
    HUD_GetStudioModelInterface   : function (_para1:longint; _para2:r_studio_interface_ss; _para3:engine_studio_api_s):longint;cdecl;
    ClientFactory                 : function :dword;cdecl;
  end;

type
  FxArray = Array[0..2] of Array[0..4] of Single;

//record Adaption of CStudioModelRenderer
type
  StudioModelRenderer_s = ^StudioModelRenderer_t;
  StudioModelRenderer_t = record
    Create                    : procedure;cdecl;
    Init                      : procedure;cdecl;
    StudioDrawModel           : function(flags: Integer):Integer;cdecl;
    StudioDrawPlayer          : function(flags: Integer; pplayer: entity_state_s): Integer;cdecl;
    StudioGetAnim             : function(m_pSubModel: model_s; pseqdesc: Pointer): Pointer;cdecl;
    StudioSetUpTransform      : procedure(trivial_accept: Integer);cdecl;
    StudioSetupBones          : procedure;cdecl;
    StudioCalcAttachments     : procedure;cdecl;
    StudioSaveBones           : procedure;cdecl;
    StudioMergeBones          : procedure(m_pSubModel: model_s);cdecl;
    StudioEstimateInterpolant : function: Single;cdecl;
    StudioEstimateFrame       : function(pseqdesc: Pointer): Single;cdecl;
    StudioFxTransform         : procedure(ent: cl_entity_s; transform: FxArray);
    StudioSlerpBones          : procedure;cdecl;//Never Call, typedef not complete
    StudioCalcBoneAdj         : procedure;cdecl;//Never Call  typedef not complete
    StudioCalcBoneQuaterion   : procedure;cdecl;//Never Call  typedef not complete
    StudioCalcBonePosition    : procedure;cdecl;//Never Call  typedef not complete
    StudioCalcRotations       : procedure;cdecl;//Never Call  typedef not complete
    StudioRenderModel         : procedure;cdecl;
    StudioRenderFinal         : procedure;cdecl;
    StudioRenderFinal_Software: procedure;cdecl;
    StudioRenderFinal_Hardware: procedure;cdecl;
    StudioPlayerBlend         : procedure(pseqdesc: Pointer; pBlend: Pinteger;pPitch: PSingle);cdecl;
    StudioEstimateGait        : procedure(pplayer: entity_state_s);
    StudioProcessGait         : procedure(pplayer: entity_state_s);
  end;

implementation

end.
